/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * HS POP3 lite Protocol Module Demo application
 */
#include "stdafx.h"
#include "list.h"
#include <commctrl.h>
#include <Commdlg.h>
#include "resource.h"
#include <stdlib.h>
#include <direct.h>
#include <stdio.h>
#include <process.h>
#include <Windowsx.h>
#include <Mmsystem.h>
#include "hspop3_if.h"
#include "hsdhcp_if.h"
#include "hsip_if.h"
#include "hssock_if.h"
#include "pcap.h"

void PutLV(HWND lvhan, hs_pop3_msg_t *msg, int i);
static int GetSelectedMessage(HWND hDlg, int *i);

#define WM_APP_TIMER	WM_USER + 1		// timer messages

#define MAXTIMERS		256

typedef struct
{
	int					allocated;
	int					enabled;
	void 				*arg;
	hs_sock_timer_cb_t 	*cb;
	UINT				timer_id;
} timerbock_t;

static timerbock_t	timers[MAXTIMERS] = {0};

// pcap local interface device list
pcap_if_t *alldevs = NULL;
pcap_t *adhandle = NULL;

unsigned char ascii_hex_eth[19] = {0};
unsigned char ethaddr[6];
unsigned char source_ipaddr[17] = {0};
unsigned char gw_ipaddr[17] = {0};
unsigned char mask[17] = {0};

static TIMECAPS timecaps;


TCHAR			szWindowClass[]		= " ";
HINSTANCE		hInst;
TCHAR			szTitle[] = "HsTCPIPv4 C Source Library: POP3 Demo";

HWND			hwnd_main;
HWND			dlg_main;
HBRUSH			hbrush1				= NULL;

timer_callback_t *pop3_timer_callback = NULL;

char commercial_msg[] = "This is a demo application for HsTCPIPv4 embedded protocol stack. HS POP3 is part of HsTCPIPv4.\n"
                        "It allows a programmer to receive email text messages from remote POP3 server.\n"
                        "Further information: www.hillstone-software.com/hs_tcpipv4_details.htm\n email: info@hillstone-software.com";

int session_open = 0;
static int msgrx_state = MSG_RXST_NONE;

long num_msgs;
long mbx_size;
long rx_size;
long old_msgs;

long session_handle;

int ip_port = DEF_PORT;
unsigned char server_name[256] = "pop3.server.com";
unsigned char username[256] = "user1";
unsigned char password[256] = "password1";

hs_pop3_session_t	ss = {0};
hs_pop3_api_t       api = {0};

unsigned char port_str[5] = {0};
unsigned char regdata[1024] = {0};

conf_t cfg = 
{
	{DEF_PORT},				// port
	{0xC0A80101},			// dest ip address
	{"pop3.server.com"},
};

static COLORREF color;
static int progress = 0;

// currently received message descriptor
static hs_pop3_msg_t  	sMsg;
static HANDLE			hFile = NULL;

static unsigned char	*pMsgText = NULL;

// allocate timer
static timerbock_t *alloc_timer(void)
{
	int i;

	for (i=0; i<MAXTIMERS; i++)
	{
		if (!timers[i].allocated)
		{
			timers[i].allocated = TRUE;
			return &timers[i];
		}
	}

	return NULL;
}

// free timer
static void free_timer(timerbock_t *p)
{
	p->allocated = FALSE;
}

static unsigned char hex2byte(unsigned char *buf)
{
	unsigned char hex[] = "0123456789ABCDEF";
	unsigned char b = 0;
    unsigned char b1, b2;
	int i;

    b1 = toupper(buf[0]);
    b2 = toupper(buf[1]);

	for (i = 0; i<16; i++)
	{
		if (b1 == hex[i])
		{
			b &= 0x0f;
			b = (unsigned char)((i << 4) & 0xf0);
			break;
		}
	}

	for (i = 0; i<16; i++)
	{
		if (b2 == hex[i])
		{
			b &= 0xf0;
			b |= (unsigned char)(i & 0x0f);
			break;
		}
	}
	
	return b;
}

void write_event(char *str)
{
	SetDlgItemText(dlg_main, IDC_STATIC_STATUS, str);
}

// Convert ascii hex ethernet address in the form 00:00:00:00:00:00 to binary 6 byte buffer
static void convert_hex_ethaddr_to_binary(unsigned char *hexbuf, unsigned char *binbuf)
{
	int i, n, maxlen, done;

	maxlen = 17;

	if (strlen(hexbuf) != 17)
		return;

	done = FALSE;
	i = 0; n = 0;
	while (!done)
	{
		binbuf[i] = hex2byte(&hexbuf[n]);

		i++;
		n += 3;

		if (i == 6) 
			done = TRUE;
	}
}

// multimedia timer callback
void CALLBACK mmTimeProc(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	PostMessage(hwnd_main, WM_APP_TIMER, 0, (LPARAM)dwUser);
}

// Process timer message
static void process_timer(LPARAM lParam)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)lParam;

	if (!pTmrBlk) return;

	if (!pTmrBlk->allocated)
		return;

	if (!pTmrBlk->enabled)
		return;

	pTmrBlk->cb(pTmrBlk->arg);

	free_timer(pTmrBlk);
}

// start timer function
static long sock_timer_start(unsigned long timeout_ms, void *arg, hs_sock_timer_cb_t *cb)
{
	MMRESULT res;
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *)alloc_timer();
	if (!pTmrBlk) return 0;

	pTmrBlk->arg = arg;
	pTmrBlk->cb = cb;
	pTmrBlk->enabled = TRUE;

	res = timeSetEvent((UINT)timeout_ms, timecaps.wPeriodMin, mmTimeProc, (DWORD)pTmrBlk, TIME_ONESHOT);
	if (res == 0) 
	{	
		free_timer(pTmrBlk);
		return 0;
	}

	pTmrBlk->timer_id = (UINT)res;

	return (long)pTmrBlk;
}

// stop timer function
static void sock_timer_stop(long timer_han)
{
	timerbock_t *pTmrBlk;

	pTmrBlk = (timerbock_t *) timer_han;

	if (!pTmrBlk) return;

	if (!pTmrBlk->enabled) return;

	timeKillEvent(pTmrBlk->timer_id);
	pTmrBlk->enabled = FALSE;
}

// transmit over physical layer (ethernet)
static void sock_drv_tx_fn(unsigned char *pkt, int len)
{
	if (!adhandle) return;

	/* Send down the packet */
	if (pcap_sendpacket(adhandle,	// Adapter
		pkt,						// buffer with the packet
		len							// size
		) != 0)
	{
		return;
	}
}

// callback used by HSTCP to get number of millisecond ticks since bootup
// Used for generating pseudo random initial sequence number
// if callback pointer not initialized, HSTCP will use default 
// pseudo random number generator to produce initial sequence numbers
static unsigned long hs_sock_get_ticks(void)
{
	return (unsigned long)GetTickCount();
}

// Callback used by the stack whenever it needs a random 32 bit number
static unsigned long hs_sock_get_rand32(void)
{
	return (unsigned long)rand();
}

// global socket events
static long hs_sock_global_event(long user_handle, int ev, long arg1, long arg2, long arg3, long arg4)
{
	hs_dhcp_update_t *p;
	char str[17];

	switch (ev)
	{

	// have been aquired by HSDHCP
	case HS_SOCK_EV_DHCP_UPDATE:
		p = (hs_dhcp_update_t *)arg1;
		if (!p) return 0;

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->local_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS1, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->dns_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_DNS, str);		

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->router_ip, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_GW, str);

		memset(str, 0, sizeof(str));
		HsSockInetNtoa(p->subnet_mask, str);
		SetDlgItemText(dlg_main, IDC_IPADDRESS_MASK, str);

		break;
	}

	return 0;
}

HWND NewDialogWin(HINSTANCE hInst, HWND parent, DLGPROC proc, int dlg_id)
{
	HWND	handle;

	handle = CreateDialog(hInst, (LPCTSTR)dlg_id, parent, proc);

	ShowWindow	(handle, SW_SHOWDEFAULT);
    UpdateWindow(handle);

	return	handle;
}

ATOM BlRegisterClass(HINSTANCE	hInstance,
					UINT		style,
					WNDPROC		proc,
					int			cursor_id,
					HBRUSH		bkg_brush,
					LPCTSTR		lpszClassName,
					int			icon_id,
					int			menu_id)
{
	WNDCLASSEX	wcex;

	wcex.cbSize			= sizeof(WNDCLASSEX); 

	wcex.style			= style;
	wcex.lpfnWndProc	= proc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= 0;

	wcex.hCursor		= 0;
	if (cursor_id)
	{
		wcex.hCursor	= LoadCursor(0, (const char *)cursor_id);
	}

	wcex.hbrBackground	= bkg_brush;
	wcex.lpszMenuName	= (LPCSTR)menu_id;
	wcex.lpszClassName	= lpszClassName;

	wcex.hIconSm		= 0; 
	if (icon_id)
	{
		wcex.hIcon		= LoadIcon(hInstance, (LPCTSTR)icon_id);
	}
	return RegisterClassEx(&wcex);
}

/*
 * Retrieve configuration from registry, if not found create
 */
void get_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp, size;

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					"Software\\Hillstone Software\\HS POP3 C Source Library Demo Application\\1.1",
            		0, KEY_QUERY_VALUE, &hKey );

    if (res == ERROR_SUCCESS)
    {    
		RegQueryValueEx(hKey, "RegData", 0, &dwDisp, (LPBYTE) regdata, &size);

   		memcpy(&cfg, regdata, sizeof(conf_t));
   		ip_port = cfg.port;
   		
		memset(server_name, 0, sizeof server_name);
		memcpy(server_name, cfg.srvname, sizeof cfg.srvname);

		memset(username, 0, sizeof username);
		memcpy(username, cfg.username, sizeof cfg.username);

		memset(password, 0, sizeof password);
		memcpy(password, cfg.password, sizeof password);

    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Hillstone Software\\HS POP3 C Source Library Demo Application\\1.1", 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{	
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

/*
 * Update configuration in registry
 */
void update_config(void)
{
	LONG	res;
	HKEY	hKey = NULL;

	HKEY hk; 
   	DWORD dwDisp;
	BOOL rs = FALSE;

	SendMessage(GetDlgItem(dlg_main, IDC_IPADDRESS2), IPM_GETADDRESS, 0,(LPARAM)&cfg.dest_ip);

	cfg.port = (WORD)GetDlgItemInt(dlg_main, IDC_EDIT_PORT, &rs, FALSE);
	if (!rs) ip_port = DEF_PORT;

	memset(cfg.srvname, 0, sizeof cfg.srvname);
	GetDlgItemText(dlg_main, IDC_EDIT_SRVNAME, cfg.srvname, (sizeof cfg.srvname) - 1);	

	memset(cfg.username, 0, sizeof cfg.username);
    GetDlgItemText(dlg_main, IDC_EDIT_USERNAME, cfg.username, (sizeof cfg.username) - 1);

	memset(cfg.password, 0, sizeof cfg.password);
    GetDlgItemText(dlg_main, IDC_EDIT_PASSWORD, cfg.password, (sizeof cfg.password) - 1);
	

	/* Check if our registry key exists */
	res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
					"Software\\Hillstone Software\\HS POP3 C Source Library Demo Application\\1.1",
            		0, KEY_ALL_ACCESS, &hKey );

    if (res == ERROR_SUCCESS)
    {    
   		memcpy(regdata, &cfg, sizeof(conf_t));
   		RegSetValueEx(hKey, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hKey);
	}
	else if (RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\Hillstone Software\\HS POP3 C Source Library Demo Application\\1.1", 
          	0, NULL, REG_OPTION_NON_VOLATILE, KEY_WRITE, NULL, &hk, &dwDisp)) 
   	{
   		return;
   	}
   	else
   	{
   		memcpy(regdata, &cfg, sizeof(conf_t));
		RegSetValueEx(hk, "RegData", 0, REG_BINARY, (LPBYTE) regdata, sizeof(conf_t));		
    	RegCloseKey(hk);
	}
}

/* check if a message with given message id string is already in the received list */
static int message_id_exists(unsigned char *id)
{
	int n, i;
    LVITEM item = {0};
	unsigned char str[80];	
	HWND lvhan = GetDlgItem(dlg_main, IDC_LIST1);

	n = ListView_GetItemCount(lvhan);
	for (i=0; i<n; i++)
	{
		memset(str, 0, sizeof (str));
		ListView_GetItemText(lvhan, i, 3, str, sizeof(str));
		
		if (strcmp(id, str) == 0)
			return TRUE;		
	}
	return FALSE;
}

/* save next message block */
static int save_next_msg_block(HANDLE hFile, unsigned char *buf, int len)
{
	DWORD written;
	
	WriteFile(hFile, (LPCVOID)buf, len, &written, NULL);

	if ((int)written != len)
		return FALSE;

	sMsg.dwMsgLen += written;

	return TRUE;
}

static void rollback_saved_message(void)
{
	if (msgrx_state == MSG_RXST_RXMSG)
	{
		SetFilePointer(hFile, (long)(-(long)(sMsg.dwMsgLen + sizeof(hs_pop3_msg_t))), NULL, FILE_CURRENT);
		SetEndOfFile(hFile);
	}	
}

/* Event callback from POP3 module */
int pop3_callback(long user_data, int ev, long arg1, long arg2)
{
	unsigned char s[300] = {0};
	unsigned char *errstr = (unsigned char *)arg1;
	int len = (int)arg2;
	int i;
    hs_pop3_msg_t *msg;
	unsigned char *p;
	long llen;
	int rc = TRUE;
	DWORD written;

	switch (ev)
	{
	// message id
	case HS_POP3_USR_EV_MSGID:
		p = (unsigned char *)arg1;

		if (message_id_exists(p))
		{
			old_msgs++;
			return FALSE;	
		}		

		msgrx_state = MSG_RXST_RXHDR;
		break;

	// message header
	case HS_POP3_USR_EV_DONE_MSGHDR:
		msg = (hs_pop3_msg_t *)arg1;

		memcpy(&sMsg, msg, sizeof(hs_pop3_msg_t));

		WriteFile(hFile, (LPCVOID)msg, sizeof(hs_pop3_msg_t), &written, NULL);

		sMsg.dwMsgLen = 0;
		msgrx_state = MSG_RXST_RXMSG;
		break;

	// next message block
	case HS_POP3_USR_EV_DONE_MSGBLK:		
		save_next_msg_block(hFile, (unsigned char *)arg1, (int)arg2);
		break;

	// current message received
    case HS_POP3_USR_EV_DONE_MSG:		
		SetFilePointer(hFile, (long)(-(long)(sMsg.dwMsgLen + sizeof(hs_pop3_msg_t))), NULL, FILE_CURRENT);
		WriteFile(hFile, (LPCVOID)&sMsg, sizeof(hs_pop3_msg_t), &written, NULL);
		SetFilePointer(hFile, 0, NULL, FILE_END); 

		PutLV(GetDlgItem(dlg_main, IDC_LIST1), &sMsg, 0xffff);
		
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "message received");

		msgrx_state = MSG_RXST_NONE;
        break;

	case HS_POP3_USR_EV_DONE:
		session_open = 0;

		rollback_saved_message();

		msgrx_state = MSG_RXST_NONE;
	
        SendMessage(GetDlgItem(dlg_main, IDC_PROGRESS1), PBM_SETPOS, (WPARAM)100, 0);
		color = GREEN;
        if (arg1 > 0)
        {
            sprintf(s, "MAIL SESSION COMPLETE ( %u ) MESSAGES ( %u ) NEW", arg1, arg1 - old_msgs);
        }
        else
        {
            sprintf(s, "MAIL SESSION COMPLETE (MAIL BOX EMPTY)");
        }
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
		break;

	case HS_POP3_USR_EV_TIMEDOUT:
		sprintf(s, "ERROR: TIMEOUT: (%s)", errstr);		
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);

		rollback_saved_message();

		msgrx_state = MSG_RXST_NONE;
		break;

	case HS_POP3_USR_EV_INTERR:
		session_open = 0;
		memset(s, 0, sizeof(s));
		sprintf(s, "Internal error: %s", (unsigned char *)arg1);
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);

		rollback_saved_message();

		msgrx_state = MSG_RXST_NONE;
		break;

	case HS_POP3_USR_EV_SRVERR:
		i = sprintf(s, "ERROR: Server Returns Error: ");
        if (((sizeof s) - i-1) < (size_t)len) len = ((sizeof s) - i-1);		
		memcpy(&s[i], errstr, len);
        p = strrchr(s, '\r'); if (p) *p = 0;
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);

		rollback_saved_message();

		msgrx_state = MSG_RXST_NONE;
		break;

	case HS_POP3_USR_EV_CONNFAIL:
		session_open = 0;
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "ERROR: TCP CONNECTION REQUEST FAILED");

		msgrx_state = MSG_RXST_NONE;
		break;

	case HS_POP3_USR_EV_CLOSED:
		session_open = 0;
		color = RED;
		SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "ERROR: TCP SESSION DISCONNECTED");

		rollback_saved_message();

		msgrx_state = MSG_RXST_NONE;
		break;
		
	case HS_POP3_USR_EV_PROGRESS1:
		num_msgs = arg1;
		mbx_size = arg2;        
		break;

	case HS_POP3_USR_EV_PROGRESS2:
		llen = arg2;
		rx_size += llen;
		
		progress = (rx_size * 100) / mbx_size;
		SendMessage(GetDlgItem(dlg_main, IDC_PROGRESS1), PBM_SETPOS, (WPARAM)progress, 0);
		break;
	}

	return rc;
}


static void LvInsertCol(HWND hwndListView, int index, unsigned char *hdr, int cx, int fmt)
{
    LV_COLUMN lvColumn = {0};
    
    lvColumn.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM;
   	lvColumn.pszText = hdr;
	lvColumn.cx = cx;
	lvColumn.fmt = fmt;
    ListView_InsertColumn (hwndListView, index, &lvColumn);    
}

static void InitLV(HWND hwndListView)
{
  	// Empty the list in list view.
  	ListView_DeleteAllItems (hwndListView);

  	// Insert the five columns in the list view.
    LvInsertCol(hwndListView, 0, "FROM", 200, LVCFMT_LEFT);
    LvInsertCol(hwndListView, 1, "SUBJ", 100, LVCFMT_LEFT);
    LvInsertCol(hwndListView, 2, "DATE", 100, LVCFMT_LEFT);
    LvInsertCol(hwndListView, 3, "MSGID", 250, LVCFMT_LEFT);

	ListView_SetExtendedListViewStyle(hwndListView, /*LVS_EX_GRIDLINES | */
		                             LVS_EX_FULLROWSELECT);
}

void PutLV(HWND lvhan, hs_pop3_msg_t *msg, int i)
{
	LVITEM item = {0};
	unsigned char s[80] = {0};
	int index;

	memset(&item, 0, sizeof (LVITEM));
		
	item.mask = LVIF_TEXT | LVIF_PARAM;
	
	item.pszText = msg->from;
	item.lParam = (LPARAM)1;
	item.iItem = i;

    index = ListView_InsertItem(lvhan, (LPLVITEM)&item);

	ListView_SetItemText(lvhan, index, 1, msg->subj);
	ListView_SetItemText(lvhan, index, 2, msg->date);
	ListView_SetItemText(lvhan, index, 3, msg->msgid);
}

static int GetSelectedMessage(HWND hDlg, int *i)
{
	LRESULT iSelect;
    LVITEM item = {0};

	if (ListView_GetItemCount(GetDlgItem(hDlg, IDC_LIST1)) == 0)
		return FALSE;

	iSelect=SendMessage(GetDlgItem(hDlg, IDC_LIST1), LVM_GETNEXTITEM, -1, LVNI_FOCUSED);
	
	*i = (int)iSelect;

	return TRUE;
}

void delete_message(HWND hDlg)
{
    int i;

	if (!GetSelectedMessage(hDlg, &i))
		return;	

	if (MessageBox(hDlg, "Delete selected msg?", "Confirm delete", 
		MB_YESNO | MB_ICONWARNING) != IDYES)
		return;

	DeleteSelectedMessage(i);

	SetDlgItemText(hDlg, IDC_EDIT_MSG, "");
}

// Initialize
static void initialize(HWND hDlg)
{
	int rc, inum, i;

	unsigned char ascii_hex_eth[19] = {0};
	unsigned char ethaddr[6];

	unsigned char source_ipaddr[17] = {0};
	unsigned char gw_ipaddr[17] = {0};
	unsigned char mask[17] = {0};
	unsigned char dns_ipaddr[17] = "0.0.0.0";

	char str[256];

	pcap_if_t *d;

	char errbuf[PCAP_ERRBUF_SIZE];

	hssock_init_t sockinit = {0};

	if (adhandle)
		pcap_close(adhandle);
			
	inum = (int)SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_GETCURSEL, 0, 0);
	if (inum == CB_ERR)
	{
		write_event("Local adapter not selected from list");
		return;
	}

	GetDlgItemText(hDlg, IDC_IPADDRESS1, source_ipaddr, sizeof (source_ipaddr) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_GW, gw_ipaddr, sizeof (gw_ipaddr) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_MASK, mask, sizeof (mask) - 1);
	GetDlgItemText(hDlg, IDC_IPADDRESS_DNS, dns_ipaddr, sizeof (dns_ipaddr) - 1);		
	GetDlgItemText(hDlg, IDC_EDIT_MAC, ascii_hex_eth, sizeof (ascii_hex_eth) - 1);

	convert_hex_ethaddr_to_binary(ascii_hex_eth, ethaddr);
				
	/* Jump to the selected adapter */
	for(d=alldevs, i=0; i< inum ;d=d->next, i++);

	if ((adhandle= pcap_open_live(d->name,	// name of the device
					 65536,					// portion of the packet to capture. 
											// 65536 grants that the whole packet will be captured on all the MACs.
					 1,						// promiscuous mode (nonzero means promiscuous)
					 1,						// read timeout
					 errbuf					// error buffer
					 )) == NULL)
	{
		write_event("Unable to open the adapter : not supported by WinPcap");
		return;
	}

	sockinit.dhcp_client = (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED) ? 1: 0;

	if (sockinit.dhcp_client)
		strcpy(source_ipaddr, "0.0.0.0");

	sockinit.source_ip_str = source_ipaddr;
	sockinit.gateway_ip_str = gw_ipaddr;
	sockinit.network_mask = mask;
	sockinit.dns_server_ip_str = dns_ipaddr;	
	sockinit.local_eth = ethaddr;
	sockinit.timer_start_fn = sock_timer_start;
	sockinit.timer_stop_fn = sock_timer_stop;
	sockinit.drv_tx_fn = sock_drv_tx_fn;
	sockinit.log_fn = write_event;

	sockinit.get_ticks_fn = hs_sock_get_ticks;
	sockinit.seed = (unsigned long)GetTickCount();
	sockinit.get_rand32 = hs_sock_get_rand32;
	sockinit.global_ev_fn = hs_sock_global_event;

	rc = HsSockInit(&sockinit);
	if (rc != HS_SOCK_RC_OK)
	{
		sprintf(str, "HsSockInit failed, rc=%s", HsSockGetRcString(rc));
		write_event(str);
		return;
	}

    api.start_timer = sock_timer_start;
    api.stop_timer = sock_timer_stop;
    api.event_cb = pop3_callback;

    rc = HsPop3Init(&api);
	if (rc != HS_POP3_RC_OK)
	{
        unsigned char s[100] = {0};
        sprintf(s, "HS POP3 Initialisation failure code [%d]", rc);
        
		write_event(s);
		return;
	}

	write_event("HsTCPIPv4 Initialized");
}

// Shut down HsTCPIPv4
static void shut_down(HWND hDlg)
{
	session_open = 0;
	HsPop3Destroy();
	HsSockCleanUp();
			
	if (adhandle)
	{
		pcap_close(adhandle);
		adhandle = NULL;
	}

	write_event("HsTCPIPv4 De-Initialized");
}

/*
 * Main dialog handler
 */
LRESULT CALLBACK DlgWin(HWND hDlg, UINT message, 
						  WPARAM wParam, LPARAM lParam)
{
	LRESULT result = FALSE;
	int rc;
    char windir[_MAX_PATH] = {0};
    char expl_path[_MAX_PATH] = {0};
	int i;

	unsigned char ascii_hex_eth[19] = {0};

	unsigned char source_ipaddr[17] = {0};
	unsigned char gw_ipaddr[17] = {0};
	unsigned char dns_ipaddr[17] = {0};
	unsigned char mask[17] = {0};

	char str[256];

	pcap_if_t *d;

	char errbuf[PCAP_ERRBUF_SIZE];

	hssock_init_t sockinit = {0};
	
	switch (message)
	{
	case WM_INITDIALOG:
		dlg_main = hDlg;

		SetDlgItemText(hDlg, IDC_IPADDRESS1, "192.168.1.51");
		SetDlgItemText(hDlg, IDC_EDIT_MAC, "00:1e:68:a4:fb:15");

		SetDlgItemText(hDlg, IDC_IPADDRESS_GW, "192.168.1.254");
		SetDlgItemText(hDlg, IDC_IPADDRESS_MASK, "255.255.255.0");

		SetDlgItemText(hDlg, IDC_IPADDRESS_DNS, "0.0.0.0");
		
		CheckDlgButton(hDlg, IDC_CHECK_DHCP, BST_CHECKED);
		EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), FALSE);
		
		SendMessage(GetDlgItem(hDlg, IDC_BM_RECV), BM_SETIMAGE, IMAGE_BITMAP, 
				   (LPARAM) LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP1)));

		SendMessage(GetDlgItem(hDlg, IDC_BM_ABORT), BM_SETIMAGE, IMAGE_BITMAP, 
				   (LPARAM) LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP2)));

		SendMessage(GetDlgItem(hDlg, IDC_BM_EXIT), BM_SETIMAGE, IMAGE_BITMAP, 
				   (LPARAM) LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP3)));

		SendMessage(GetDlgItem(hDlg, IDC_BM_BUY), BM_SETIMAGE, IMAGE_BITMAP, 
				   (LPARAM) LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP4)));

		SendMessage(GetDlgItem(hDlg, IDC_BM_INFO), BM_SETIMAGE, IMAGE_BITMAP, 
				   (LPARAM) LoadBitmap(hInst, MAKEINTRESOURCE(IDB_BITMAP5)));        

        SendDlgItemMessage(hDlg, IDC_EDIT_SRVNAME, EM_LIMITTEXT, MAX_SRVNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_USERNAME, EM_LIMITTEXT, MAX_USERNAME, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_PASSWORD, EM_LIMITTEXT, MAX_PASSWORD, 0);
        SendDlgItemMessage(hDlg, IDC_EDIT_PORT, EM_LIMITTEXT, 4, 0);

        
        SetDlgItemInt(hDlg, IDC_EDIT_PORT, ip_port, FALSE);
        SetDlgItemText(hDlg, IDC_EDIT_SRVNAME, server_name);
        SetDlgItemText(hDlg, IDC_EDIT_USERNAME, username);
        
		SendDlgItemMessage(hDlg, IDC_STATIC_CM, WM_SETTEXT, 0, (long)commercial_msg);

        InitLV(GetDlgItem(hDlg, IDC_LIST1));

		load_messages();

		/* Retrieve the device list */
		if(pcap_findalldevs(&alldevs, errbuf) == -1)
		{
			sprintf(str, "Error in pcap_findalldevs: %s", errbuf);
			write_event(str);
			return TRUE;
		}

		i = 0;
		for(d=alldevs; d; d=d->next)
		{			
			sprintf(str, "[%d] %s", ++i, d->description);
			SendDlgItemMessage(hDlg, IDC_COMBO_ETHIF, CB_ADDSTRING, 0, (long)str);			
		}	

		SendMessage(GetDlgItem(hDlg, IDC_COMBO_ETHIF), CB_SETCURSEL, 0, 0);

		srand( (unsigned)time( NULL ) );		
		return TRUE;

	case WM_CTLCOLORSTATIC:
		if (((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_CM)) || 
            ((HWND)lParam == GetDlgItem(hDlg, IDC_EDIT_MSG)))
		{
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		else if ((HWND)lParam == GetDlgItem(hDlg, IDC_STATIC_STATUS))
		{
			SetTextColor((HDC)wParam, color);
			SetBkColor  ((HDC)wParam, WHITE);
			result = (long)hbrush1;
		}
		break;

	case WM_NOTIFY:
		switch ((int)(wParam))
		{
		case IDC_LIST1:
			switch (((LPNMHDR)lParam)->code)
			{
			case NM_CLICK:
				if (!GetSelectedMessage(hDlg, &i))
					break;

				LoadSelectedMsgText(i);
				break;
			}
			break;
		}
		break;

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_CHECK_DHCP:
			if (IsDlgButtonChecked(hDlg, IDC_CHECK_DHCP) == BST_CHECKED)
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), FALSE);
			else
				EnableWindow(GetDlgItem(hDlg, IDC_IPADDRESS1), TRUE);
			break;
		
		case IDC_INIT:
			initialize(hDlg);
			break;

		case IDC_SHUTDOWN:
			shut_down(hDlg);
			break;

        case IDC_BM_ABORT: 
        	if (!session_open) break;

			rc = HsPop3Abort(session_handle);
			if (rc != HS_POP3_RC_OK)
			{
				unsigned char s[200];
				sprintf(s, "ERROR: HS POP3 ERROR (%s)", HsPop3GetErrStr(rc));
				color = RED;
				SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
				break;
			}			
            break;
            
		case IDC_BM_SEND: 
			if (session_open) break;			
		
            GetDlgItemText(hDlg, IDC_EDIT_SRVNAME, ss.srv_name, (sizeof ss.srv_name) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_PORT, port_str, 4);
            ss.srv_port = (unsigned short)atoi(port_str);

            GetDlgItemText(hDlg, IDC_EDIT_USERNAME, ss.username, (sizeof ss.username) - 1);
            GetDlgItemText(hDlg, IDC_EDIT_PASSWORD, ss.password, (sizeof ss.password) - 1);
			
			color = 0;
			SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "POP3 MAIL SESSION IN PROGRESS");

			progress = 0;
			SendMessage(GetDlgItem(dlg_main, IDC_PROGRESS1), PBM_SETPOS, (WPARAM)progress, 0);

			rx_size = 0;
			old_msgs = 0;

			ss.delete_msgs = (IsDlgButtonChecked(hDlg, IDC_CHECK1) == BST_CHECKED) ? 1 : 0;

			rc = HsPop3GetMail(&ss, &session_handle);
			if (rc != HS_POP3_RC_OK)
			{
				unsigned char s[200];
				sprintf(s, "ERROR: HS POP3 ERROR (%s)", HsPop3GetErrStr(rc));
				color = RED;
				SetDlgItemText(dlg_main, IDC_STATIC_STATUS, s);
				break;
			}
			session_open = 1;
			msgrx_state = MSG_RXST_RXHDR;
			break;

		case IDC_BT_DELETE:
			delete_message(hDlg);
			break;
		}
		
		break; 
	}
    return result;
}

/*
 * Main window message handler
 */
LRESULT CALLBACK WndProc(HWND		hWnd, 
						 UINT		message, 
						 WPARAM		wParam, 
						 LPARAM		lParam)
{
    LVITEM item = {0};

	static INITCOMMONCONTROLSEX icex;

	switch (message) 
	{
	case WM_CREATE:
		icex.dwSize = sizeof(icex);
		icex.dwICC = ICC_DATE_CLASSES;

		InitCommonControlsEx(&icex);

		InitCommonControls(); 

		dlg_main = NewDialogWin(hInst, hWnd, DlgWin, IDD_DIALOG_MAIN);
		break;

	case WM_APP_TIMER:
		process_timer(lParam);
		break;

	case WM_DESTROY:
		if (hbrush1)
		{
			SelectObject(GetWindowDC(hWnd), hbrush1);
			DeleteObject(hbrush1);
		}

		HsPop3Destroy();

		HsSockCleanUp();
			
		if (alldevs)
		{
			pcap_freealldevs(alldevs);
			alldevs = NULL;
		}

		if (adhandle)
		{
			pcap_close(adhandle);
			adhandle = NULL;
		}

		update_config();

		PostQuitMessage(0);
		break;

	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
   
	return 0;
}

/*
 * Init Application Instance
 */
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow, HWND *hwnd)
{
	HWND hWnd;

	hInst = hInstance;

	hWnd = CreateWindow(szWindowClass, 
					   szTitle, 
					   WS_OVERLAPPED | WS_SYSMENU,
					   0, 
					   0, 
					   MAINWIDTH,
					   MAINDEPTH, 
					   NULL, LoadMenu(hInstance, 0/*(LPCTSTR)IDR_MENU1*/), 
					   hInstance, NULL);

	if (!hWnd)
	{
		return FALSE;
	}
	*hwnd = hWnd;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	return TRUE;
}

// load existing messages from disk file into received messages list
static void load_messages(void)
{
	HWND lvhan = GetDlgItem(dlg_main, IDC_LIST1);
	hs_pop3_msg_t m;
	DWORD dwBytes;

	SetFilePointer(hFile, 0, NULL, FILE_BEGIN);

	color = BLUE;
	SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "Loading stored messages...");

	while (1)
	{
		ReadFile(hFile, &m, sizeof(hs_pop3_msg_t), &dwBytes, NULL);
		if (dwBytes != sizeof(hs_pop3_msg_t))
			break;

		PutLV(lvhan, &m, 0xffff);

		SetFilePointer(hFile, (long)m.dwMsgLen, NULL, FILE_CURRENT);
	}

	SetFilePointer(hFile, 0, NULL, FILE_END); 

	color = GREEN;
	SetDlgItemText(dlg_main, IDC_STATIC_STATUS, "Stored messages loaded.");
}

/*
 * Search through mail file and find a message with given message id string
 * read in message header from the file. File pointer left just after message header
 */
static int find_message_in_file_by_msgid(
	unsigned char *id, 
	HANDLE 		hFile,
	hs_pop3_msg_t *pHdr, 
	DWORD		 *pFp)
{
	DWORD dwBytes;
	DWORD fp = 0;

	SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
	
	while (1)
	{
		ReadFile(hFile, pHdr, sizeof(hs_pop3_msg_t), &dwBytes, NULL);
		if (dwBytes != sizeof(hs_pop3_msg_t))
			break;

		if (strcmp(id, pHdr->msgid) == 0)
		{
			*pFp = fp;
			return TRUE;							
		}

		fp = SetFilePointer(hFile, (long)pHdr->dwMsgLen, NULL, FILE_CURRENT);
	}

	return FALSE;
}

static void LoadSelectedMsgText(int i)
{
	HWND lvhan = GetDlgItem(dlg_main, IDC_LIST1);
	hs_pop3_msg_t m;
	DWORD dwBytes;
	DWORD fp;

	unsigned char id[80];
	
	memset(id, 0, sizeof(id));
	ListView_GetItemText(lvhan, i, 3, id, sizeof(id)-1);

	if (strlen(id) == 0) 
		return;

	if (!find_message_in_file_by_msgid(id, hFile, &m, &fp))
	{
		SetFilePointer(hFile, 0, NULL, FILE_END); 
		return;
	}

	if (pMsgText)
	{
		free(pMsgText);
		pMsgText = NULL;
	}

	pMsgText = malloc(m.dwMsgLen+1);
	if (!pMsgText)
	{
		SetFilePointer(hFile, 0, NULL, FILE_END);
		return;
	}

	memset(pMsgText, 0, m.dwMsgLen+1);
	ReadFile(hFile, pMsgText, m.dwMsgLen, &dwBytes, NULL);
	if (m.dwMsgLen != dwBytes)
	{
		SetFilePointer(hFile, 0, NULL, FILE_END);
		return;				
	}

	SetDlgItemText(dlg_main, IDC_EDIT_MSG, &pMsgText[m.hdrlen]);
	SetFilePointer(hFile, 0, NULL, FILE_END);
	return;							
}

/*
 * Delete message
 */
static void DeleteSelectedMessage(int i)
{
	HWND lvhan = GetDlgItem(dlg_main, IDC_LIST1);
	hs_pop3_msg_t m;
	DWORD dwBytes, fp, size_to_read, size_to_write, read_so_far;
	HANDLE hFile1;
	unsigned char *pBuf;

	unsigned char id[80];
	
	memset(id, 0, sizeof(id));
	ListView_GetItemText(lvhan, i, 3, id, sizeof(id)-1);

	if (strlen(id) == 0) 
		return;

	if (!find_message_in_file_by_msgid(id, hFile, &m, &fp))
	{
		SetFilePointer(hFile, 0, NULL, FILE_END); 
		return;
	}
	
	hFile1 = CreateFile(".\\hspop3.new", 		// file to open
                   GENERIC_WRITE,               // open for writing and writing
                   0,       					// no sharing
                   NULL,                  		// default security
                   OPEN_ALWAYS, 	      		// overrwrite existing file
                   FILE_ATTRIBUTE_NORMAL, 		// normal file
                   NULL); 

	if (hFile1 == INVALID_HANDLE_VALUE)
	{		
		return;		
	}

	pBuf = (unsigned char *)malloc(16384);
	if (!pBuf) return;

	SetFilePointer(hFile, 0, NULL, FILE_BEGIN);
	
	read_so_far = 0;
	while (read_so_far != fp)
	{
		size_to_read = ((read_so_far + 16384) > fp) ? fp : 16384;
		
		ReadFile(hFile, pBuf, size_to_read, &dwBytes, NULL);
		if (size_to_read != dwBytes)
		{
			free(pBuf);
			SetFilePointer(hFile, 0, NULL, FILE_END); 
			CloseHandle(hFile1);
			return;		
		}

		WriteFile(hFile1, (LPCVOID)pBuf, size_to_read, &dwBytes, NULL);
		if (dwBytes != size_to_read)
		{
			free(pBuf);
			SetFilePointer(hFile, 0, NULL, FILE_END); 
			CloseHandle(hFile1);
			return;		
		}

		read_so_far += size_to_read;		
	}

	SetFilePointer(hFile, (long)(m.dwMsgLen + (sizeof(hs_pop3_msg_t))), NULL, FILE_CURRENT);

	while (1)
	{		
		ReadFile(hFile, pBuf, 16384, &dwBytes, NULL);
		if (dwBytes != 0)
		{
			size_to_write = dwBytes;
			WriteFile(hFile1, (LPCVOID)pBuf, size_to_write, &dwBytes, NULL);
			if (dwBytes != size_to_write)
			{
				free(pBuf);
				SetFilePointer(hFile, 0, NULL, FILE_END); 
				CloseHandle(hFile1);
				return;		
			}			
		}
		else
		{
			break;
		}

		if (size_to_write != 16384)
			break;
	}

	CloseHandle(hFile);
	CloseHandle(hFile1);

	DeleteFile(".\\hspop3.old");
	
	if (!MoveFile(".\\hspop3.dta", ".\\hspop3.old"))
	{
		hFile = CreateFile(".\\hspop3.dta",		// file to open
                   GENERIC_WRITE | GENERIC_READ, // open for writing and writing
                   0,       					// no sharing
                   NULL,                  		// default security
                   OPEN_ALWAYS, 	      		// overrwrite existing file
                   FILE_ATTRIBUTE_NORMAL, 		// normal file
                   NULL);

		free(pBuf);

		if (hFile == INVALID_HANDLE_VALUE)
		{
			hFile = NULL;
			return;
		}
		return;
	}

	if (!MoveFile(".\\hspop3.new", ".\\hspop3.dta"))
	{
		hFile = CreateFile(".\\hspop3.dta",		// file to open
                   GENERIC_WRITE | GENERIC_READ, // open for writing and writing
                   0,       					// no sharing
                   NULL,                  		// default security
                   OPEN_ALWAYS, 	      		// overrwrite existing file
                   FILE_ATTRIBUTE_NORMAL, 		// normal file
                   NULL);

		free(pBuf);

		if (hFile == INVALID_HANDLE_VALUE)
		{
			hFile = NULL;
			return;
		}
		return;
	}
		
	free(pBuf);

	ListView_DeleteItem(lvhan, i);

	hFile = CreateFile(".\\hspop3.dta",		// file to open
                  GENERIC_WRITE | GENERIC_READ, // open for writing and writing
                   0,       					// no sharing
                   NULL,                  		// default security
                   OPEN_ALWAYS, 	      		// overrwrite existing file
                   FILE_ATTRIBUTE_NORMAL, 		// normal file
                   NULL);
	if (hFile == INVALID_HANDLE_VALUE)
	{
		hFile = NULL;
		MessageBox(dlg_main, "Cannot re-open mailbox file!", "Error", MB_ICONERROR);
	}
}


/*
 * entry point
 */
int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG		msg;
	HACCEL	hAccelTable;
	unsigned char *pkt_data;
	struct pcap_pkthdr *header;
	int r;

	hbrush1 = CreateSolidBrush(RGB(255, 255, 255));

	get_config();

	hFile = CreateFile(".\\hspop3.dta", 		// file to open
                   GENERIC_WRITE | GENERIC_READ,// open for writing and writing
                   0,       					// no sharing
                   NULL,                  		// default security
                   OPEN_ALWAYS, 	      		// overrwrite existing file
                   FILE_ATTRIBUTE_NORMAL, 		// normal file
                   NULL); 

	if (hFile == INVALID_HANDLE_VALUE)
	{
		MessageBox(0, "Cannot open mail file", 0, 0);
		return 0;		
	}

	//SetFilePointer(hFile, 0, NULL, FILE_END); 

	BlRegisterClass(hInstance, CS_HREDRAW | CS_VREDRAW, WndProc, (int)IDC_ARROW,
				   (HBRUSH)(COLOR_APPWORKSPACE+1), szWindowClass, IDI_ICON1, 0/*IDR_MENU1*/);

	if (!InitInstance (hInstance, nCmdShow, &hwnd_main)) 
	{
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)0);

	// main message processing loop
	while (1) 
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&msg, NULL, 0, 0))
			{
				if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
				{
					TranslateMessage(&msg);
					DispatchMessage(&msg);
				}
			}
			else
			{
				break;
			}
		}
		else
		{
			Sleep(1);

			if (adhandle)
			{
				r = pcap_next_ex( adhandle, &header, &pkt_data);
				if (r > 0)
				{
					HsIpReceivePacket(pkt_data, (int)header->caplen);					
				}
			}			
		}		
	}

	if (hFile)
	{
		CloseHandle(hFile);
		hFile = NULL;
	}

	if (pMsgText)
	{
		free(pMsgText);
		pMsgText = NULL;
	}

	if (alldevs)
		pcap_freealldevs(alldevs);

	if (adhandle)
		pcap_close(adhandle);

	return msg.wParam;	
}


